import mariadb
import json
from pathlib import Path
from time import perf_counter
from pprint import pprint
from itertools import chain, repeat

conn = mariadb.connect(
       user="root",
       password="root",
       host="localhost",
       port=3306,
    )

c = conn.cursor()
c.execute('USE imdb')

with open('enums.json', encoding='utf-8') as enums_f:
    enums = json.load(enums_f)
for key in enums:  # transform to lookup table for ids: enums['genre']['Comedy'] => genre.id
    enums[key] = {name: i for i, name in enumerate(enums[key], 1)}


def idstrip(idstr, prefix_len=2):
    return int(idstr.strip()[prefix_len:])

def id_helper(parts, column, col_name, prefix_len):
    return idstrip(parts[column[col_name]], prefix_len)

def value_helper(parts, column, col_name, cls):
    cv = parts[column[col_name]].strip()
    return cls(cv) if cv != r'\N' else None

def enum_helper(parts, column, col_name, enum_name):
    v = value_helper(parts, column, col_name, str)
    return enums[enum_name][v] if v is not None else None

# valhelp and idhelp uses the parts and column directly from current scope
valhelp = lambda col_name, cls=str: value_helper(parts, column, col_name, cls)
idhelp = lambda  col_name, prefix_len=2: id_helper(parts, column, col_name, prefix_len)
enmhelp = lambda col_name, enum_name: enum_helper(parts, column, col_name, enum_name)

chunk_size = 1000
chunk = []

def insert_chunk():
    qmarks = ','.join(repeat('(?,?,?,?)', len(chunk)))
    c.execute(f'INSERT IGNORE INTO episode (title_id, parent_title_id, season, episode) VALUES {qmarks}',
              tuple(chain.from_iterable(chunk)))
    chunk.clear()


#c.execute('TRUNCATE TABLE episode')  # (only for repeated cell running in notebook)
t1 = perf_counter()
with open(Path('data') / 'title.episode.tsv', encoding='utf-8') as f:
    column = {}
    for i, p in enumerate(f.readline().strip().split('\t')):
        column[p.strip()] = i
    for line in f:
        parts = line.strip().split('\t')
        t_id = idhelp('tconst')  # ommit prefix
        tp_id = idhelp('parentTconst')  # ommit prefix
        season = valhelp('seasonNumber', int)
        episode = valhelp('episodeNumber', int)
        chunk.append((t_id, tp_id, season, episode))
        if len(chunk) > chunk_size:
            insert_chunk()

if len(chunk):
    insert_chunk()

print(perf_counter() - t1, 's')
